local L = LibStub("AceLocale-3.0"):GetLocale("DirectionArrow", false)

-- Config options
DirectionArrowConfig = { 
  type='group',
  name = L["DirectionArrow"],
  desc = L["Display an arrow that points to a raid/party member"],
  args = {
    General = {
      order = 1,
      type = "group",
      name = L["General Options"],
      desc = L["General Options"],
      args = {
        size = {
          order = 1,
          type = 'range',
          name = L["Size"],
          desc = L["The size of the arrow"],
          get = function ()
            return DirectionArrow.db.profile.size
          end,
          set = function (i, arg)
            DirectionArrow.db.profile.size = arg
            DirectionArrow.f:SetScale (arg)
          end,
          min = 1,
          max = 5,
          step = 0.1,
        },
        relative_location = {
          order = 2,
          type = 'select',
          name = L["Arrow Location"],
          desc = L["The relative location of the arrow against the unit frame."],
          get = function ()
            return DirectionArrow.db.profile.relative_location
          end,
          set = function (i, arg)
            DirectionArrow.db.profile.relative_location = arg
          end,
          values = {
            TOPLEFT = L["Top Left"],
            TOP = L["Top"],
            TOPRIGHT = L["Top Right"],
            LEFT = L["Left"],
            CENTER = L["Center"],
            RIGHT = L["Right"],
            BOTTOMLEFT = L["Bottom Left"],
            BOTTOM = L["Bottom"],
            BOTTOMRIGHT = L["Bottom Right"],
          },
        },
        spacer11 = {
          order = 3,
          type = "description",
          name = "",
        },
        opacity = {
          order = 4,
          type = 'range',
          name = L["Opacity"],
          desc = L["The opacity of the arrow"],
          get = function ()
            return DirectionArrow.db.profile.opacity
          end,
          set = function (i, arg)
            DirectionArrow.db.profile.opacity = arg
            DirectionArrow.f:SetAlpha (arg)
          end,
          min = 0,
          max = 1,
          step = 0.05,
        },
        update_frequency = {
          order = 5,
          type = 'range',
          name = L["Update Frequency"],
          desc = L["The number of cycles between each arrow update. 0 means every cycle (Possibly CPU intensive)"],
          get = function ()
            return DirectionArrow.db.profile.update_frequency
          end,
          set = function (i, arg)
            DirectionArrow.db.profile.update_frequency = arg
          end,
          min = 0,
          max = 1,
          step = 0.05,
        },
        spacer12 = {
          order = 6,
          type = "description",
          name = "",
        },
        show_distance = {
          order = 7,
          type = 'toggle',
          name = L["Show Distance"],
          desc = L["Show distance text to unit "],
          get = function ()
            return DirectionArrow.db.profile.show_distance
          end,
          set = function (i, arg)
            DirectionArrow.db.profile.show_distance = arg
            if arg then
              DirectionArrow.x:Show()
            else
              DirectionArrow.x:Hide()
            end
          end
        },
        distance_size = {
          order = 8,
          type = 'range',
          name = L["Distance Size"],
          desc = L["The size of the distance text"],
          disabled = function ()
            return not DirectionArrow.db.profile.show_distance
          end,
          get = function ()
            return DirectionArrow.db.profile.distance_size
          end,
          set = function (i, arg)
            DirectionArrow.db.profile.distance_size = arg
            DirectionArrow.x:SetFont(DirectionArrow.x:GetFont(), arg)
            DirectionArrow.x2:SetFont(DirectionArrow.x:GetFont(), arg)
          end,
          min = 0,
          max = 10,
          step = 0.1,
        },
        spacer13 = {
          order = 9,
          type = "description",
          name = "",
        },
        distance_color = {
          order = 10,
          type = 'color',
          name = L["Distance text color"],
          desc = L["The color of the distance text"],
          disabled = function ()
            return not DirectionArrow.db.profile.show_distance
          end,
          get = function ()
            return unpack (DirectionArrow.db.profile.distance_color)
          end,
          set = function (i, r, g, b, a)
            DirectionArrow.db.profile.distance_color = {r, g, b, a}
            DirectionArrow.x:SetTextColor (r, g, b)
          end,
          hasAlpha = false,
        },
        distance_ds_color = {
          order = 11,
          type = 'color',
          name = L["Distance text drop shadow color"],
          desc = L["The color of the distance text drop shadow"],
          disabled = function ()
            return not DirectionArrow.db.profile.show_distance
          end,
          get = function ()
            return unpack (DirectionArrow.db.profile.distance_ds_color)
          end,
          set = function (i, r, g, b, a)
            DirectionArrow.db.profile.distance_ds_color = {r, g, b, a}
            DirectionArrow.x2:SetTextColor (r, g, b)
          end,
          hasAlpha = false,
        },
--[===[@debug@
        spacer199 = {
          order = 99,
          type = "description",
          name = "",
        },
        debug = {
          order = 100,
          type = 'toggle',
          name = "Debug",
          desc = "Debug",
          get = function ()
            return DirectionArrow.db.profile.debug
          end,
          set = function (i, arg)
            DirectionArrow.db.profile.debug = arg
          end
        },
--@end-debug@]===]
      }
    }
  }
}

-- Config defaults
DirectionArrowDefaultConfigDB = {
  global = {
    ["*"] = {},
  },
  profile = {
    size = 1,
    opacity = 1,
    relative_location = "CENTER",
    update_frequency = 0,
    show_distance = true,
    distance_size = 4,
    distance_color = {1, 0, 0},
    distance_ds_color = {0, 0, 0},
--[===[@debug@
    debug = true,
--@end-debug@]===]
  }
}
